from django.conf import settings


def find_related_models(section):
    rel = None
    relateds = section._meta.get_all_related_objects()
    for related in relateds:
        found = "%s.%s" % (related.model.__module__,
                related.model.__name__)
        if found == settings.ARMSTRONG_SECTION_ITEM_MODEL:
            rel = related
            break
    kwargs = {rel.field.name: section}
    qs = rel.model.objects.filter(**kwargs)
    if hasattr(qs, 'select_subclasses'):
        qs = qs.select_subclasses()
    return qs



from django.conf import settings
from django.db import models
from mptt.models import MPTTModel
from mptt.fields import TreeForeignKey


class SectionManager(models.Manager):
    def get(self, **kwargs):
        defaults = {}
        defaults.update(kwargs)
        if 'full_slug' in defaults:
            if defaults['full_slug'] and defaults['full_slug'][-1] != "/":
                defaults['full_slug'] += "/"
        return super(SectionManager, self).get(**defaults)


class Section(MPTTModel):
    title = models.CharField(max_length=255)
    summary = models.TextField(default="", blank=True)
    slug = models.SlugField()
    full_slug = models.CharField(max_length=255, blank=True)

    parent = TreeForeignKey('self', null=True, blank=True)

    objects = SectionManager()

    @property
    def items(self):
        backend = getattr(settings, "ARMSTRONG_SECTION_ITEM_BACKEND",
                "armstrong.core.arm_sections.backends.find_related_models")
        module, func = backend.rsplit('.', 1)
        m = __import__(module, globals(), locals(), module)
        return getattr(m, func)(self)

    def save(self, *args, **kwargs):
        orig_full_slug = self.full_slug
        if self.parent:
            self.full_slug = "%s%s/" % (self.parent.full_slug, self.slug)
        else:
            self.full_slug = "%s/" % self.slug
        obj = super(Section, self).save(*args, **kwargs)
        if orig_full_slug != self.full_slug:
            for child in self.get_children():
                child.save()
        return obj

    def __unicode__(self):
        return "%s (%s)" % (self.title, self.full_slug)



from django.contrib import admin
from mptt.admin import MPTTModelAdmin
from mptt.forms import TreeNodeChoiceField
from mptt.forms import TreeNodeMultipleChoiceField
from mptt.models import MPTTModel

from .models import Section

EMPTY_LABEL = u="---------"


class SectionTreeAdminMixin(object):
    """
    Provides a mixin for dealing with tree display of Section <select>s in
    admin forms
    """

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if issubclass(db_field.rel.to, MPTTModel):
            required = db_field.formfield().required
            return TreeNodeChoiceField(queryset=db_field.rel.to.objects.all(),
                    required=required, empty_label=EMPTY_LABEL)
        return super(SectionTreeAdminMixin, self).formfield_for_foreignkey(db_field,
                request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if issubclass(db_field.rel.to, MPTTModel):
            required = db_field.formfield().required
            return TreeNodeMultipleChoiceField(empty_label=EMPTY_LABEL,
                    queryset=db_field.rel.to.objects.all(), required=required)
        return super(SectionTreeAdminMixin, self).formfield_for_manytomany(db_field,
                request, **kwargs)


class SectionAdmin(MPTTModelAdmin):
    exclude = ("full_slug", )


admin.site.register(Section, SectionAdmin)
